using System;
using System.Drawing;
using System.Text;

using ABB.Robotics.Tps.Windows.Forms;
using ABB.Robotics.Controllers;
using ABB.Robotics.Controllers.IOSystemDomain;
using ABB.Robotics.Controllers.RapidDomain;
using ABB.Robotics.Diagnostics;
using ABB.Robotics.Tps.Resources;

namespace TpsViewSampleApp
{
    public class ViewProduction : TpsForm
    {

        #region Fields

        //layout
        private GroupBox groupBox1;
        private GroupBox groupBox2;
        private System.Windows.Forms.Label _lblCurProgram;
        private System.Windows.Forms.Label _lblActive;
        private TpsLabel _lblActiveValue;
        private System.Windows.Forms.Label _lblCycles;
        private ABB.Robotics.Tps.Windows.Forms.Button _btnStop;
        private ABB.Robotics.Tps.Windows.Forms.Button _btnStartRout;
        private ABB.Robotics.Tps.Windows.Forms.Button _btnGoHome;
        private ABB.Robotics.Tps.Windows.Forms.Button _btnStartNewBatch;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem1;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem2;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem_TRob1;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem_TRob2;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem_TRob3;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem_TRob4;
        private ABB.Robotics.DataBinding.RapidDataBindingSource rapidDataBindingSource1;

        //Resource manager, used to fetch images
        private TpsResourceManager _tpsRm = null;

        //ABB CAPI
        private Controller _controller = null;
        private Task _taskTROB1 = null;

        private ABB.Robotics.Controllers.RapidDomain.RapidData _rdSimulated = null; //boolean used to simulate process 
        private ABB.Robotics.Controllers.IOSystemDomain.DigitalSignal _doActive = null;

        private const string _module = "User";
        private const string _moduleName = "MainModule";
        private const string _serviceRoutName = "DoProcessService";
        private string _programName = string.Empty;
        private float _doActiveValue = 0;

        // flag to avoid multiple subscriptions to controller events or deactivating
        // subscriptions that are already deactivated
        private bool _isSubscribing = false;
        private System.Windows.Forms.PictureBox pictureBox1;
        private System.Windows.Forms.Label label1;
        private System.Windows.Forms.Label _lblCurProgramValue;
        private System.Windows.Forms.PictureBox _picGun;
        private System.Windows.Forms.Label label2;
        private System.Windows.Forms.PictureBox pictureBox2;
        private System.Windows.Forms.Label _lblCyclesValue;

        #endregion

        /// <summary>
        /// Required designer variable.
        /// </summary>

        #region Constructor

        /// <summary>
        /// Creates a new Production view instance 
        /// </summary>
        /// <param name="contr"></param>
        public ViewProduction(Controller contr, TpsResourceManager rM)
        {
            _controller = contr;
            _tpsRm = rM;

            // layout
            InitializeComponent();
            menuItem2.Click += new EventHandler(menuItem2_Click);
            menuItem_TRob1.Click += new EventHandler(menuItem_TRob1_Click);
            menuItem_TRob2.Click += new EventHandler(menuItem_TRob2_Click);
            menuItem_TRob3.Click += new EventHandler(menuItem_TRob3_Click);
            menuItem_TRob4.Click += new EventHandler(menuItem_TRob4_Click);

            // Get TROB_1 task defined in the rapid domain
            _taskTROB1 = _controller.Rapid.GetTask("T_ROB1");

            // create I/O signal "doActive
            _doActive = (DigitalSignal)_controller.IOSystem.GetSignal("doActive");
            if (_doActive == null) // (If signal does not exist, the return value is null)
            {
                throw new ABB.Robotics.GeneralException("Process signal doActive is missing.", "ViewProduction Constructor", string.Empty, string.Empty);
            }

            // create RAPID data 'simulated'
            _rdSimulated = _controller.Rapid.GetRapidData("T_ROB1", _module, "simulated");
        }

        #endregion

        #region Dispose

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing)
        {
            try
            {
                if (!IsDisposed)
                {
                    if (disposing)
                    {
                        Trace.Write("Production dispose executing...\n");
                        // Dispose rapidDataBindingSource1,necessary to avoid memory leak!
                        if (rapidDataBindingSource1 != null)
                        {
                            rapidDataBindingSource1.Dispose();
                            rapidDataBindingSource1 = null;
                        }
                        // Dispose of any CAPI object
                        if (_taskTROB1 != null)
                        {
                            _taskTROB1.Dispose();
                            _taskTROB1 = null;
                        }
                        if (_doActive != null)
                        {
                            if (_isSubscribing == true)
                            {
                                _doActive.Changed -= new SignalChangedEventHandler(_doActive_Changed);
                            }
                            _doActive.Dispose();
                            _doActive = null;
                        }

                        if (_rdSimulated != null)
                        {
                            _rdSimulated.Dispose();
                            _rdSimulated = null;
                        }
                    }
                    base.Dispose(disposing);
                }
            }
            catch (System.Exception e)
            {
                ExceptionHandler.ReThrowException("ViewProduction", "Dispose", e);
            }
        }
        #endregion

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            ABB.Robotics.DataBinding.RapidDataObject rapidDataObject3 = new ABB.Robotics.DataBinding.RapidDataObject();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(ViewProduction));
            this.menuItem1 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.menuItem_TRob1 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.menuItem_TRob2 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.menuItem_TRob3 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.menuItem_TRob4 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.groupBox2 = new ABB.Robotics.Tps.Windows.Forms.GroupBox();
            this.rapidDataBindingSource1 = new ABB.Robotics.DataBinding.RapidDataBindingSource();
            this._lblCyclesValue = new System.Windows.Forms.Label();
            this._lblCurProgramValue = new System.Windows.Forms.Label();
            this._picGun = new System.Windows.Forms.PictureBox();
            this.pictureBox1 = new System.Windows.Forms.PictureBox();
            this.label1 = new System.Windows.Forms.Label();
            this._lblActiveValue = new ABB.Robotics.Tps.Windows.Forms.TpsLabel();
            this._lblCycles = new System.Windows.Forms.Label();
            this._lblActive = new System.Windows.Forms.Label();
            this._lblCurProgram = new System.Windows.Forms.Label();
            this._btnStartRout = new ABB.Robotics.Tps.Windows.Forms.Button();
            this._btnGoHome = new ABB.Robotics.Tps.Windows.Forms.Button();
            this.menuItem2 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.groupBox1 = new ABB.Robotics.Tps.Windows.Forms.GroupBox();
            this.label2 = new System.Windows.Forms.Label();
            this.pictureBox2 = new System.Windows.Forms.PictureBox();
            this._btnStop = new ABB.Robotics.Tps.Windows.Forms.Button();
            this._btnStartNewBatch = new ABB.Robotics.Tps.Windows.Forms.Button();
            this.groupBox2.SuspendLayout();
            this.groupBox1.SuspendLayout();
            this.SuspendLayout();
            // 
            // menuItem1
            // 
            this.menuItem1.Checked = false;
            this.menuItem1.DockToRight = false;
            this.menuItem1.Enabled = true;
            this.menuItem1.Image = null;
            this.menuItem1.ImageSelected = null;
            this.menuItem1.Index = 0;
            this.menuItem1.MenuItems.Add(this.menuItem_TRob1);
            this.menuItem1.MenuItems.Add(this.menuItem_TRob2);
            this.menuItem1.MenuItems.Add(this.menuItem_TRob3);
            this.menuItem1.MenuItems.Add(this.menuItem_TRob4);
            this.menuItem1.Pressed = false;
            this.menuItem1.Text = "Rapid Tasks";
            this.menuItem1.Width = 128;
            // 
            // menuItem_TRob1
            // 
            this.menuItem_TRob1.Checked = false;
            this.menuItem_TRob1.DockToRight = false;
            this.menuItem_TRob1.Enabled = true;
            this.menuItem_TRob1.Image = null;
            this.menuItem_TRob1.ImageSelected = null;
            this.menuItem_TRob1.Index = 0;
            this.menuItem_TRob1.Pressed = false;
            this.menuItem_TRob1.Text = "T_ROB1";
            this.menuItem_TRob1.Width = 128;
            // 
            // menuItem_TRob2
            // 
            this.menuItem_TRob2.Checked = false;
            this.menuItem_TRob2.DockToRight = false;
            this.menuItem_TRob2.Enabled = true;
            this.menuItem_TRob2.Image = null;
            this.menuItem_TRob2.ImageSelected = null;
            this.menuItem_TRob2.Index = 1;
            this.menuItem_TRob2.Pressed = false;
            this.menuItem_TRob2.Text = "T_ROB2";
            this.menuItem_TRob2.Width = 128;
            // 
            // menuItem_TRob3
            // 
            this.menuItem_TRob3.Checked = false;
            this.menuItem_TRob3.DockToRight = false;
            this.menuItem_TRob3.Enabled = true;
            this.menuItem_TRob3.Image = null;
            this.menuItem_TRob3.ImageSelected = null;
            this.menuItem_TRob3.Index = 2;
            this.menuItem_TRob3.Pressed = false;
            this.menuItem_TRob3.Text = "T_ROB3";
            this.menuItem_TRob3.Width = 128;
            // 
            // menuItem_TRob4
            // 
            this.menuItem_TRob4.Checked = false;
            this.menuItem_TRob4.DockToRight = false;
            this.menuItem_TRob4.Enabled = true;
            this.menuItem_TRob4.Image = null;
            this.menuItem_TRob4.ImageSelected = null;
            this.menuItem_TRob4.Index = 3;
            this.menuItem_TRob4.Pressed = false;
            this.menuItem_TRob4.Text = "T_ROB4";
            this.menuItem_TRob4.Width = 128;
            // 
            // groupBox2
            // 
            this.groupBox2.BackColor = System.Drawing.Color.LightGray;
            this.groupBox2.Controls.Add(this._lblCyclesValue);
            this.groupBox2.Controls.Add(this._lblCurProgramValue);
            this.groupBox2.Controls.Add(this._picGun);
            this.groupBox2.Controls.Add(this.pictureBox1);
            this.groupBox2.Controls.Add(this.label1);
            this.groupBox2.Controls.Add(this._lblActiveValue);
            this.groupBox2.Controls.Add(this._lblCycles);
            this.groupBox2.Controls.Add(this._lblActive);
            this.groupBox2.Controls.Add(this._lblCurProgram);
            this.groupBox2.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this.groupBox2.HorizontalAlign = ABB.Robotics.Tps.Windows.Forms.HorAlign.Left;
            this.groupBox2.LineStyle = ABB.Robotics.Tps.Windows.Forms.LineStyle.SingleLine;
            this.groupBox2.Location = new System.Drawing.Point(12, 31);
            this.groupBox2.Name = "groupBox2";
            this.groupBox2.Size = new System.Drawing.Size(615, 159);
            this.groupBox2.Title = "Process status T_ROB1";
            this.groupBox2.VerticalAlign = ABB.Robotics.Tps.Windows.Forms.VerAlign.Center;
            // 
            // rapidDataBindingSource1
            // 
            this.rapidDataBindingSource1.DataSource = typeof(ABB.Robotics.DataBinding.RapidDataObject);
            rapidDataObject3.ModuleName = "user";
            rapidDataObject3.TaskName = "T_ROB1";
            rapidDataObject3.VariableName = "cycles";
            this.rapidDataBindingSource1.RapidDataList.Add(rapidDataObject3);
            // 
            // _lblCyclesValue
            // 
            this._lblCyclesValue.DataBindings.Add(new System.Windows.Forms.Binding("Text", this.rapidDataBindingSource1, "Value", true, System.Windows.Forms.DataSourceUpdateMode.OnPropertyChanged));
            this._lblCyclesValue.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Bold);
            this._lblCyclesValue.Location = new System.Drawing.Point(254, 108);
            this._lblCyclesValue.Name = "_lblCyclesValue";
            this._lblCyclesValue.Size = new System.Drawing.Size(155, 24);
            this._lblCyclesValue.Text = "_lblCyclesValue";
            // 
            // _lblCurProgramValue
            // 
            this._lblCurProgramValue.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Bold);
            this._lblCurProgramValue.Location = new System.Drawing.Point(254, 33);
            this._lblCurProgramValue.Name = "_lblCurProgramValue";
            this._lblCurProgramValue.Size = new System.Drawing.Size(341, 20);
            this._lblCurProgramValue.Text = "_lblCurProgramValue";
            // 
            // _picGun
            // 
            this._picGun.Image = ((System.Drawing.Image)(resources.GetObject("_picGun.Image")));
            this._picGun.Location = new System.Drawing.Point(307, 56);
            this._picGun.Name = "_picGun";
            this._picGun.Size = new System.Drawing.Size(45, 45);
            // 
            // pictureBox1
            // 
            this.pictureBox1.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox1.Image")));
            this.pictureBox1.Location = new System.Drawing.Point(394, 64);
            this.pictureBox1.Name = "pictureBox1";
            this.pictureBox1.Size = new System.Drawing.Size(30, 30);
            this.pictureBox1.Visible = false;
            // 
            // label1
            // 
            this.label1.Font = new System.Drawing.Font("Tahoma", 11F, System.Drawing.FontStyle.Italic);
            this.label1.Location = new System.Drawing.Point(434, 76);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(139, 20);
            this.label1.Text = "Simulated mode!";
            this.label1.Visible = false;
            // 
            // _lblActiveValue
            // 
            this._lblActiveValue.BackColor = System.Drawing.Color.LightGray;
            this._lblActiveValue.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this._lblActiveValue.Location = new System.Drawing.Point(256, 56);
            this._lblActiveValue.Size = new System.Drawing.Size(51, 45);
            this._lblActiveValue.TabIndex = 16;
            this._lblActiveValue.TextAlignment = System.Drawing.ContentAlignment.TopLeft;
            this._lblActiveValue.Title = "";
            // 
            // _lblCycles
            // 
            this._lblCycles.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Regular);
            this._lblCycles.Location = new System.Drawing.Point(3, 108);
            this._lblCycles.Name = "_lblCycles";
            this._lblCycles.Size = new System.Drawing.Size(264, 18);
            this._lblCycles.Text = "Number of batch items produced:";
            // 
            // _lblActive
            // 
            this._lblActive.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Regular);
            this._lblActive.Location = new System.Drawing.Point(3, 70);
            this._lblActive.Name = "_lblActive";
            this._lblActive.Size = new System.Drawing.Size(195, 20);
            this._lblActive.Text = "Currently processing:";
            // 
            // _lblCurProgram
            // 
            this._lblCurProgram.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Regular);
            this._lblCurProgram.Location = new System.Drawing.Point(3, 33);
            this._lblCurProgram.Name = "_lblCurProgram";
            this._lblCurProgram.Size = new System.Drawing.Size(247, 20);
            this._lblCurProgram.Text = "Loaded RAPID program:";
            // 
            // _btnStartRout
            // 
            this._btnStartRout.BackColor = System.Drawing.SystemColors.ActiveBorder;
            this._btnStartRout.BackgroundImage = null;
            this._btnStartRout.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this._btnStartRout.Location = new System.Drawing.Point(471, 81);
            this._btnStartRout.Size = new System.Drawing.Size(129, 54);
            this._btnStartRout.TabIndex = 12;
            this._btnStartRout.Text = "Start Routine";
            this._btnStartRout.TextAlign = ABB.Robotics.Tps.Windows.Forms.ContentAlignmentABB.MiddleCenter;
            this._btnStartRout.Click += new System.EventHandler(this._btnStartServiceRout_Click);
            // 
            // _btnGoHome
            // 
            this._btnGoHome.BackColor = System.Drawing.SystemColors.ActiveBorder;
            this._btnGoHome.BackgroundImage = null;
            this._btnGoHome.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this._btnGoHome.Location = new System.Drawing.Point(471, 20);
            this._btnGoHome.Size = new System.Drawing.Size(129, 55);
            this._btnGoHome.TabIndex = 13;
            this._btnGoHome.Text = "Home";
            this._btnGoHome.TextAlign = ABB.Robotics.Tps.Windows.Forms.ContentAlignmentABB.MiddleCenter;
            this._btnGoHome.Click += new System.EventHandler(this._btnGoHome_Click);
            // 
            // menuItem2
            // 
            this.menuItem2.Checked = false;
            this.menuItem2.DockToRight = true;
            this.menuItem2.Enabled = true;
            this.menuItem2.Image = null;
            this.menuItem2.ImageSelected = null;
            this.menuItem2.Index = 1;
            this.menuItem2.Pressed = false;
            this.menuItem2.Text = "Close";
            this.menuItem2.Width = 128;
            // 
            // groupBox1
            // 
            this.groupBox1.BackColor = System.Drawing.Color.LightGray;
            this.groupBox1.Controls.Add(this.label2);
            this.groupBox1.Controls.Add(this.pictureBox2);
            this.groupBox1.Controls.Add(this._btnStop);
            this.groupBox1.Controls.Add(this._btnGoHome);
            this.groupBox1.Controls.Add(this._btnStartNewBatch);
            this.groupBox1.Controls.Add(this._btnStartRout);
            this.groupBox1.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this.groupBox1.HorizontalAlign = ABB.Robotics.Tps.Windows.Forms.HorAlign.Left;
            this.groupBox1.LineStyle = ABB.Robotics.Tps.Windows.Forms.LineStyle.SingleLine;
            this.groupBox1.Location = new System.Drawing.Point(12, 196);
            this.groupBox1.Name = "groupBox1";
            this.groupBox1.Size = new System.Drawing.Size(612, 145);
            this.groupBox1.Title = "Service";
            this.groupBox1.VerticalAlign = ABB.Robotics.Tps.Windows.Forms.VerAlign.Center;
            // 
            // label2
            // 
            this.label2.Font = new System.Drawing.Font("Tahoma", 10F, System.Drawing.FontStyle.Bold);
            this.label2.Location = new System.Drawing.Point(123, 20);
            this.label2.Name = "label2";
            this.label2.Size = new System.Drawing.Size(184, 96);
            this.label2.Text = "These commands start or stop RAPID execution in any operating mode.";
            // 
            // pictureBox2
            // 
            this.pictureBox2.Image = ((System.Drawing.Image)(resources.GetObject("pictureBox2.Image")));
            this.pictureBox2.Location = new System.Drawing.Point(14, 20);
            this.pictureBox2.Name = "pictureBox2";
            this.pictureBox2.Size = new System.Drawing.Size(32, 32);
            // 
            // _btnStop
            // 
            this._btnStop.BackColor = System.Drawing.SystemColors.ActiveBorder;
            this._btnStop.BackgroundImage = null;
            this._btnStop.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this._btnStop.Location = new System.Drawing.Point(330, 20);
            this._btnStop.Size = new System.Drawing.Size(129, 55);
            this._btnStop.TabIndex = 18;
            this._btnStop.Text = "Stop";
            this._btnStop.TextAlign = ABB.Robotics.Tps.Windows.Forms.ContentAlignmentABB.MiddleCenter;
            this._btnStop.Click += new System.EventHandler(this._btnStop_Click);
            // 
            // _btnStartNewBatch
            // 
            this._btnStartNewBatch.BackColor = System.Drawing.SystemColors.ActiveBorder;
            this._btnStartNewBatch.BackgroundImage = null;
            this._btnStartNewBatch.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this._btnStartNewBatch.Location = new System.Drawing.Point(330, 81);
            this._btnStartNewBatch.Size = new System.Drawing.Size(129, 54);
            this._btnStartNewBatch.TabIndex = 16;
            this._btnStartNewBatch.Text = "Start";
            this._btnStartNewBatch.TextAlign = ABB.Robotics.Tps.Windows.Forms.ContentAlignmentABB.MiddleCenter;
            this._btnStartNewBatch.Click += new System.EventHandler(this._btnStartNewBatch_Click);
            // 
            // ViewProduction
            // 
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Inherit;
            this.BackColor = System.Drawing.Color.LightGray;
            this.Controls.Add(this.groupBox1);
            this.Controls.Add(this.groupBox2);
            // 
            // 
            // 
            this.MainMenu.MenuItems.Add(this.menuItem1);
            this.MainMenu.MenuItems.Add(this.menuItem2);
            this.Size = new System.Drawing.Size(640, 390);
            this.Text = "Production";
            this.Controls.SetChildIndex(this.groupBox2, 0);
            this.Controls.SetChildIndex(this.groupBox1, 0);
            this.groupBox2.ResumeLayout(false);
            this.groupBox1.ResumeLayout(false);
            this.ResumeLayout(false);

        }

        #endregion

        void _btnStartServiceRout_Click(object sender, EventArgs e)
        {
            try
            {
                if (_controller.Rapid.ExecutionStatus != ExecutionStatus.Running)
                {
                    // Set program pointer to service routine and start executing
                    _taskTROB1.SetProgramPointer(_moduleName, _serviceRoutName);
                    _controller.Rapid.Start();
                }
                else
                {
                    GTPUMessageBox.Show(this,null
                    , "RAPID execution must be stopped before the action can be performed."
                    , "System Information"
                    , System.Windows.Forms.MessageBoxIcon.Hand
                    , System.Windows.Forms.MessageBoxButtons.OK);
                }
            }
            catch (System.Exception)
            {
                GTPUMessageBox.Show(this, null
                , "An error occurred when trying to set program pointer to service routine and start execution. Check the event log for further information."
                , "System Error"
                , System.Windows.Forms.MessageBoxIcon.Hand
                , System.Windows.Forms.MessageBoxButtons.OK);
            }
        }

            #region Public methods used to control activation and deactivation of subscriptions to controller events

        /// <summary>
        /// To be executed after constructor & when this view is active again after Sample Application activation via task bar 
        /// </summary>
        public void Activate()
        {
            // process active?
            _doActiveValue = _doActive.Value;

            if (_doActiveValue != 0)
            {
                _lblActiveValue.Text = "YES";
                _picGun.Image = _tpsRm.GetImage("process_active.gif");
            }
            else
            {
                _lblActiveValue.Text = "NO";
                _picGun.Image = _tpsRm.GetImage("process_inactive.gif");

            }

            // Update current value of RAPID data 'simulated'
            UpdateSimulatedStatus();

            // loaded program?
            if (_taskTROB1.ProgramName != string.Empty)
            {
                // there is a loaded program, possible to retrieve name
                _programName = _taskTROB1.ProgramName;
            }
            else
            {
                _programName = "No Program";
            }
            _lblCurProgramValue.Text = _programName;

            // set up subscription to controller event
            if (_isSubscribing == false)
            {
                _doActive.Changed += new SignalChangedEventHandler(_doActive_Changed);
                _isSubscribing = true;
            }
        }

        /// <summary>
        /// To be executed if this view is active at Sample Application deactivation via FP task bar.
        /// Removes any active subscriptions toward robot controller.
        /// </summary>
        public void UnSubscribe()
        {
            if (_isSubscribing == true)
            {
                _doActive.Changed -= new SignalChangedEventHandler(_doActive_Changed);
                _isSubscribing = false;
            }
        }

        #endregion

        #region Event handlers

        /// <summary>
        /// Executes when value of I/O signal doActive in the controller has changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void _doActive_Changed(object sender, SignalChangedEventArgs e)
        {
            try
            {
                _doActiveValue = e.NewSignalState.Value;

                //enforce thread switch to perform update on GUI thread
                this.Invoke(new EventHandler(UpdateActiveLbl), sender, e);
            }
            catch (System.Exception)
            {
                Debug.Assert(false, "ActiveSignalChanged event handler failed");
            }
        }

        /// <summary>
        /// This method updates the GUI. It is guaranteed to execute on the GUI thread.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void UpdateActiveLbl(object sender, EventArgs e)
        {
            try
            {
                if (_doActiveValue != 0)
                {
                    _lblActiveValue.Text = "YES";
                    _picGun.Image = _tpsRm.GetImage("process_active.gif");
                }
                else
                {
                    _lblActiveValue.Text = "NO";
                    _picGun.Image = _tpsRm.GetImage("process_inactive.gif");
                }
            }
            catch (System.Exception ex)
            {
                Debug.Assert(false, "UpdateActiveLbl failed with message: ", ex.ToString());
            }
        }

        /// <summary>
        /// Executes when "Start New Batch" is pressed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void _btnStartNewBatch_Click(object sender, EventArgs e)
        {
            if (_controller.Rapid.ExecutionStatus != ExecutionStatus.Running)
            {
                string msg = string.Format("This batch has produced {0} items. The items counter will now be reset to zero. \nPress OK to proceed.", _lblCyclesValue.Text);

                GTPUMessageBox.Show(this.Parent
                    , new ABB.Robotics.Tps.Windows.Forms.MessageBoxEventHandler(StartAnew1) // The callback method will execute when the Message Box is closed
                    , msg
                    , "System Information"
                    , System.Windows.Forms.MessageBoxIcon.Asterisk
                    , System.Windows.Forms.MessageBoxButtons.OKCancel);
            }
            else
            {
                GTPUMessageBox.Show(this, null
                , "RAPID execution must be stopped before the action can be performed."
                , "System Information"
                , System.Windows.Forms.MessageBoxIcon.Hand
                , System.Windows.Forms.MessageBoxButtons.OK);
            }
        }

        /// <summary>
        /// Executes when the user has pressed OK or Cancel in Message box. 
        /// The method checks that OK has been pressed before launching a new message.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void StartAnew1(object sender, ABB.Robotics.Tps.Windows.Forms.MessageBoxEventArgs e)
        {
            // Perform only if user has confirmed with OK, (if Cancel do nothing)
            if (e.DialogResult == System.Windows.Forms.DialogResult.OK)
            {
                GTPUMessageBox.Show(this.Parent
                 , new ABB.Robotics.Tps.Windows.Forms.MessageBoxEventHandler(StartAnew2) // The callback method will start RAPID execution
                 , "Press OK to start a new batch. Be aware that program execution will start immediately."
                 , "System Information"
                 , System.Windows.Forms.MessageBoxIcon.Asterisk
                 , System.Windows.Forms.MessageBoxButtons.OKCancel);
            }
        }

        /// <summary>
        /// Executes when the user has pressed OK or Cancel in Message box. 
        /// The method checks that OK has been pressed before taking action: 
        /// resets PP, resets counter (writing to RAPID data in robot controller), 
        /// starts RAPID execution.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void StartAnew2(object sender, ABB.Robotics.Tps.Windows.Forms.MessageBoxEventArgs e)
        {
            try
            {
                // Perform only if user has confirmed with OK, (if Cancel do nothing)
                if (e.DialogResult == System.Windows.Forms.DialogResult.OK)
                {
                    _controller.Rapid.ResetProgramPointer();

                    // Init RapidData object -  Global RAPID data in USER.sys: PERS num cycles := 0;
                    ABB.Robotics.Controllers.RapidDomain.RapidData rdCycles = _controller.Rapid.GetRapidData("T_ROB1", _module, "cycles");

                    // Reset RAPID data value to 0
                    IRapidData ird = rdCycles.Value;
                    ird.FillFromString("0");
                    rdCycles.Value = ird;

                    // Dispose
                    rdCycles.Dispose();

                    // Start program execution
                    _controller.Rapid.Start();
                }
            }
            catch (System.Exception ex)
            {
                GTPUMessageBox.Show(this, null
                    , string.Format("An error occurred when trying to reset RAPID data 'cycles' and start RAPID execution. \nCheck the event log for information about the cause. \n\n Error message: {0} ", ex.Message)
                    , "System"
                    , System.Windows.Forms.MessageBoxIcon.Hand
                    , System.Windows.Forms.MessageBoxButtons.OK);
            }
        }

        /// <summary>
        /// Executes when "Go To Home Position" is pressed. Stops RAPID execution, sets process signal
        /// to 0, and moves robot to home position. (last part not implemented)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void _btnGoHome_Click(object sender, EventArgs e)
        {
            try
            {
                if (_controller.Rapid.ExecutionStatus != ExecutionStatus.Running)
                {
                    System.Threading.Thread.Sleep(2000);

                    // To be implemented...
                    // _taskTROB1.SetProgramPointer(moduleName, "GoHomeRoutine");
                    // _controller.Rapid.Start();

                    GTPUMessageBox.Show(this, null
                    , "Operation ready!\n\nThe robot has successfully returned to its home position."
                    , "System information "
                    , System.Windows.Forms.MessageBoxIcon.Asterisk
                    , System.Windows.Forms.MessageBoxButtons.OK);
                }
                else
                {
                    GTPUMessageBox.Show(this, null
                    , "RAPID execution must be stopped before the action can be performed."
                    , "System Information"
                    , System.Windows.Forms.MessageBoxIcon.Hand
                    , System.Windows.Forms.MessageBoxButtons.OK);
                }
            }
            catch (System.Exception ex)
            {
                GTPUMessageBox.Show(this, null
                    , string.Format("An error occurred when trying to stop execution and return to home position. {0}", ex.ToString())
                    , "System Error"
                    , System.Windows.Forms.MessageBoxIcon.Hand
                    , System.Windows.Forms.MessageBoxButtons.OK);
            }
        }

        private void _btnStop_Click(object sender, EventArgs e)
        {
            try
            {
                if (_controller.Rapid.ExecutionStatus == ExecutionStatus.Running)
                {
                    _doActive.Value = 0;
                    _controller.Rapid.Stop();
                }
            }
            catch (System.Exception ex)
            {
                GTPUMessageBox.Show(this, null
                , string.Format("An error occurred when trying to stop execution. {0} ", ex.ToString())
                , "System Error"
                , System.Windows.Forms.MessageBoxIcon.Hand
                , System.Windows.Forms.MessageBoxButtons.OK);
            }
        }

        void menuItem2_Click(object sender, EventArgs e)
        {
            this.CloseMe();
        }

        void menuItem_TRob1_Click(object sender, EventArgs e)
        {
            try
            {
                groupBox2.Title = "Current process status T_ROB1";

                if (_doActiveValue != 0)
                {
                    _lblActiveValue.Text = "YES";
                    _picGun.Image = _tpsRm.GetImage("process_active.gif");
                }
                else
                {
                    _lblActiveValue.Text = "NO";
                    _picGun.Image = _tpsRm.GetImage("process_inactive.gif");
                }

                // loaded program?
                if (_taskTROB1.ProgramName != string.Empty)
                {
                    // there is a loaded program, possible to retrieve name
                    _programName = _taskTROB1.ProgramName;
                }
                else
                {
                    _programName = "No Program";
                }
                _lblCurProgramValue.Text = _programName;
            }
            catch (System.Exception ex)
            {
                Debug.Assert(false, "menuItem_TRob1_Click failed with message: ", ex.ToString());
            }
        }
        void menuItem_TRob2_Click(object sender, EventArgs e)
        {
            groupBox2.Title = "Current process status T_ROB2";
            _lblCurProgramValue.Text = "NOT YET IMPLEMENTED";
        }
        void menuItem_TRob3_Click(object sender, EventArgs e)
        {
            groupBox2.Title = "Current process status T_ROB3";
            _lblCurProgramValue.Text = "NOT YET IMPLEMENTED";
        }
        void menuItem_TRob4_Click(object sender, EventArgs e)
        {
            groupBox2.Title = "Current process status T_ROB4";
            _lblCurProgramValue.Text = "NOT YET IMPLEMENTED";
        }

        #endregion

        #region Private methods

        private void UpdateSimulatedStatus()
        {
            try
            {
                // read RAPID data value
                bool isSimulated = Convert.ToBoolean(_rdSimulated.Value.ToString());

                if (isSimulated == false)
                {
                    this.label1.Visible = false;
                    this.pictureBox1.Visible = false;
                }
                else
                {
                    this.label1.Visible = true;
                    this.pictureBox1.Visible = true;
                }
            }
            catch (Exception ex)
            {
                GTPUMessageBox.Show(this.Parent.Parent, null
                    , string.Format("An unexpected error occurred when reading RAPID data 'simulated'. Message {0}", ex.ToString())
                    , "System Error"
                    , System.Windows.Forms.MessageBoxIcon.Hand
                    , System.Windows.Forms.MessageBoxButtons.OK);
            }
        }

        #endregion


    }
}