using System;
using System.Drawing;
using System.Text;

using ABB.Robotics.Tps.Windows.Forms;
using ABB.Robotics.Taf;
using ABB.Robotics.Tps.Resources;
using ABB.Robotics.Controllers;
using ABB.Robotics.Controllers.RapidDomain;
using ABB.Robotics.Diagnostics;

namespace TpsViewSampleApp
{
    public class ViewTest : TpsForm
    {

        #region Fields

        //layout
        private GroupBox groupBox1;
        private GroupBox groupBox2;

        private ABB.Robotics.Tps.Windows.Forms.Button _btnOff;
        private ABB.Robotics.Tps.Windows.Forms.Button _btnOn;
        private ABB.Robotics.Tps.Windows.Forms.Button _btnSimOn;
        private ABB.Robotics.Tps.Windows.Forms.Button _btnSimOff;
        private ABB.Robotics.Tps.Windows.Forms.Button _btnStartRout;


        private System.Windows.Forms.Label _lblNoSig;
        private System.Windows.Forms.Label label1;
        private System.Windows.Forms.Label label2;
        private System.Windows.Forms.Label _lblSimValue;
        private System.Windows.Forms.Label label3;
        private System.Windows.Forms.Label label4;

        private ABB.Robotics.Tps.Windows.Forms.ListView listView1;
        private ABB.Robotics.Tps.Windows.Forms.ColumnHeader columnHeader1;
        private ABB.Robotics.Tps.Windows.Forms.ColumnHeader columnHeader2;

        // represents selected list view item
        ABB.Robotics.Tps.Windows.Forms.ListViewItem _lvItem = null;

        private System.Windows.Forms.DataGrid _dataGrid;

        //ref. to robot controller
        private Controller _controller;

        //used to launch jogging and program editor & Close button
        private ABB.Robotics.Tps.Taf.ITpsViewLaunchServices _launchService;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem1;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem2;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem3;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem4;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem5;
        private ABB.Robotics.Tps.Windows.Forms.MenuItem menuItem6;
        private object _cookie; //used as an in parameter for closing the standard view that this app launched

        // used to get/set I/O signals
        private ABB.Robotics.DataBinding.SignalObject _tempObj = null;
        private ABB.Robotics.DataBinding.SignalBindingSource signalBindingSource1;

        // RAPID data used to simulate process 
        private ABB.Robotics.Controllers.RapidDomain.RapidData _rdSimulated = null;
        private const string _module = "User";

        // flag to ensure we do not set up multiple subscriptions to controller events or deactivate
        // subscriptions that are already deactivated
        private bool _isSubscribing = false;

        #endregion
        /// <summary>
        /// Required designer variable.
        /// </summary>

        #region Constructor

        /// <summary>
        /// Creates a new Test view instance
        /// </summary>
        /// <param name="ctr"></param>
        /// <param name="iTpsSite">Needed for launching standard views</param>
        public ViewTest(Controller ctr, ABB.Robotics.Tps.Taf.ITpsViewLaunchServices iTpsSite)
        {
            _controller = ctr;
            _launchService = iTpsSite;

            InitializeComponent();

            _lblNoSig.Text = signalBindingSource1.Count.ToString();

            this.menuItem2.Click += new EventHandler(menuItem2_Click);
            this.menuItem3.Click += new EventHandler(menuItem3_Click);
            this.menuItem4.Click += new EventHandler(menuItem4_Click);
            this.menuItem5.Click += new EventHandler(menuItem5_Click);
            this.menuItem6.Click += new EventHandler(menuItem6_Click);

            // Create RAPID data 'simulated'
            _rdSimulated = _controller.Rapid.GetRapidData("T_ROB1", _module, "simulated");

            // Get TROB_1 task defined in the rapid domain
            Task taskTROB1 = _controller.Rapid.GetTask("T_ROB1");

            // Get all routines of the task and add them to listview
            if (taskTROB1 != null)
            {
                Module[] rapidModules = taskTROB1.GetModules();
                if (rapidModules != null)
                {
                    for (int i = 0; i < rapidModules.Length; i++)
                    {
                        ListViewItem item;
                        Routine[] routines = rapidModules[i].GetRoutines();
                        for (int j = 0; j < routines.Length; j++)
                        {
                            item = new ListViewItem(routines[j].Name);
                            listView1.Items.Add(item);
                            item.SubItems.Add(rapidModules[i].Name);
                            routines[j].Dispose();
                        }
                        rapidModules[i].Dispose();
                    }

                    listView1.Sort();
                }
                taskTROB1.Dispose();
            }
        }

        #endregion

        #region Dispose

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        /// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
        protected override void Dispose(bool disposing)
        {
            try
            {
                if (!IsDisposed)
                {
                    if (disposing)
                    {
                        if (_tempObj != null)
                        {
                            _tempObj.Dispose();
                            _tempObj = null;
                        }
                        if (signalBindingSource1 != null)
                        {
                            signalBindingSource1.Dispose();
                            signalBindingSource1 = null;
                        }
                        Trace.Write("Disposing TEST...\n");

                        // dispose of all CAPI objects
                        if (_rdSimulated != null)
                        {
                            // Remove subscription is still active
                            if (_isSubscribing == true)
                            {
                                _rdSimulated.ValueChanged -= new DataValueChangedEventHandler(_onSimulated_ValueChanged);
                                _isSubscribing = false;
                            }
                            _rdSimulated.Dispose();
                            _rdSimulated = null;
                        }
                    }
                    base.Dispose(disposing);
                }
            }
            catch (System.Exception e)
            {
                ExceptionHandler.ReThrowException("ViewTest", "Dispose", e);
            }
        }

        #endregion

        #region Windows Form Designer generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            ABB.Robotics.DataBinding.SignalObject signalObject1 = new ABB.Robotics.DataBinding.SignalObject();
            ABB.Robotics.DataBinding.SignalObject signalObject2 = new ABB.Robotics.DataBinding.SignalObject();
            ABB.Robotics.DataBinding.SignalObject signalObject3 = new ABB.Robotics.DataBinding.SignalObject();
            ABB.Robotics.DataBinding.SignalObject signalObject4 = new ABB.Robotics.DataBinding.SignalObject();
            ABB.Robotics.DataBinding.SignalObject signalObject5 = new ABB.Robotics.DataBinding.SignalObject();
            ABB.Robotics.DataBinding.SignalObject signalObject6 = new ABB.Robotics.DataBinding.SignalObject();
            ABB.Robotics.DataBinding.SignalObject signalObject7 = new ABB.Robotics.DataBinding.SignalObject();
            ABB.Robotics.DataBinding.SignalObject signalObject8 = new ABB.Robotics.DataBinding.SignalObject();
            ABB.Robotics.DataBinding.SignalObject signalObject9 = new ABB.Robotics.DataBinding.SignalObject();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(ViewTest));
            this.listView1 = new ABB.Robotics.Tps.Windows.Forms.ListView();
            this.columnHeader1 = new ABB.Robotics.Tps.Windows.Forms.ColumnHeader();
            this.columnHeader2 = new ABB.Robotics.Tps.Windows.Forms.ColumnHeader();
            this._btnStartRout = new ABB.Robotics.Tps.Windows.Forms.Button();
            this.menuItem1 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.menuItem2 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.menuItem3 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.menuItem6 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.menuItem5 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.signalBindingSource1 = new ABB.Robotics.DataBinding.SignalBindingSource();
            this.groupBox1 = new ABB.Robotics.Tps.Windows.Forms.GroupBox();
            this.label3 = new System.Windows.Forms.Label();
            this._lblNoSig = new System.Windows.Forms.Label();
            this.label1 = new System.Windows.Forms.Label();
            this._btnOff = new ABB.Robotics.Tps.Windows.Forms.Button();
            this._dataGrid = new System.Windows.Forms.DataGrid();
            this._btnOn = new ABB.Robotics.Tps.Windows.Forms.Button();
            this._btnSimOn = new ABB.Robotics.Tps.Windows.Forms.Button();
            this._btnSimOff = new ABB.Robotics.Tps.Windows.Forms.Button();
            this.label2 = new System.Windows.Forms.Label();
            this.groupBox2 = new ABB.Robotics.Tps.Windows.Forms.GroupBox();
            this.label4 = new System.Windows.Forms.Label();
            this._lblSimValue = new System.Windows.Forms.Label();
            this.menuItem4 = new ABB.Robotics.Tps.Windows.Forms.MenuItem();
            this.groupBox1.SuspendLayout();
            this.groupBox2.SuspendLayout();
            this.SuspendLayout();
            // 
            // listView1
            // 
            this.listView1.BackColor = System.Drawing.Color.White;
            this.listView1.BorderSize = 1;
            this.listView1.CheckBoxes = true;
            this.listView1.Columns.Add(this.columnHeader1);
            this.listView1.Columns.Add(this.columnHeader2);
            this.listView1.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this.listView1.ForeColor = System.Drawing.Color.Black;
            this.listView1.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Clickable;
            this.listView1.ItemHeigth = -1;
            this.listView1.LargeImageList = null;
            this.listView1.ListViewItemSorter = null;
            this.listView1.Location = new System.Drawing.Point(232, 24);
            this.listView1.MultiSelect = false;
            this.listView1.Scrollable = true;
            this.listView1.SelectedIndex = -1;
            this.listView1.SelectionEnabledOverScrollbuttons = false;
            this.listView1.ShowNumberOfItems = true;
            this.listView1.ShowSelection = false;
            this.listView1.Size = new System.Drawing.Size(292, 117);
            this.listView1.SmallImageList = null;
            this.listView1.Sorting = ABB.Robotics.Tps.Windows.Forms.SortOrder.Ascending;
            this.listView1.TabIndex = 17;
            this.listView1.View = System.Windows.Forms.View.Details;
            this.listView1.Zoomable = false;
            // 
            // columnHeader1
            // 
            this.columnHeader1.HeaderSorter = null;
            this.columnHeader1.Text = "Routines";
            this.columnHeader1.TextAlign = System.Windows.Forms.HorizontalAlignment.Left;
            this.columnHeader1.Width = 185;
            // 
            // columnHeader2
            // 
            this.columnHeader2.HeaderSorter = null;
            this.columnHeader2.Text = "Module";
            this.columnHeader2.TextAlign = System.Windows.Forms.HorizontalAlignment.Left;
            this.columnHeader2.Width = 115;
            // 
            // _btnStartRout
            // 
            this._btnStartRout.BackColor = System.Drawing.SystemColors.ActiveBorder;
            this._btnStartRout.BackgroundImage = null;
            this._btnStartRout.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this._btnStartRout.Location = new System.Drawing.Point(530, 24);
            this._btnStartRout.Size = new System.Drawing.Size(76, 70);
            this._btnStartRout.TabIndex = 12;
            this._btnStartRout.Text = "Start";
            this._btnStartRout.TextAlign = ABB.Robotics.Tps.Windows.Forms.ContentAlignmentABB.MiddleCenter;
            this._btnStartRout.Click += new System.EventHandler(this._btnStartRout_Click);
            // 
            // menuItem1
            // 
            this.menuItem1.Checked = false;
            this.menuItem1.DockToRight = false;
            this.menuItem1.Enabled = true;
            this.menuItem1.Image = null;
            this.menuItem1.ImageSelected = null;
            this.menuItem1.Index = 0;
            this.menuItem1.MenuItems.Add(this.menuItem2);
            this.menuItem1.MenuItems.Add(this.menuItem3);
            this.menuItem1.MenuItems.Add(this.menuItem6);
            this.menuItem1.Pressed = false;
            this.menuItem1.Text = "Show View";
            this.menuItem1.Width = 128;
            // 
            // menuItem2
            // 
            this.menuItem2.Checked = false;
            this.menuItem2.DockToRight = false;
            this.menuItem2.Enabled = true;
            this.menuItem2.Image = null;
            this.menuItem2.ImageSelected = null;
            this.menuItem2.Index = 0;
            this.menuItem2.Pressed = false;
            this.menuItem2.Text = "Jogging";
            this.menuItem2.Width = 128;
            // 
            // menuItem3
            // 
            this.menuItem3.Checked = false;
            this.menuItem3.DockToRight = false;
            this.menuItem3.Enabled = true;
            this.menuItem3.Image = null;
            this.menuItem3.ImageSelected = null;
            this.menuItem3.Index = 1;
            this.menuItem3.Pressed = false;
            this.menuItem3.Text = "Program Editor";
            this.menuItem3.Width = 128;
            // 
            // menuItem6
            // 
            this.menuItem6.Checked = false;
            this.menuItem6.DockToRight = false;
            this.menuItem6.Enabled = true;
            this.menuItem6.Image = null;
            this.menuItem6.ImageSelected = null;
            this.menuItem6.Index = 2;
            this.menuItem6.Pressed = false;
            this.menuItem6.Text = "Program Data";
            this.menuItem6.Width = 128;
            // 
            // menuItem5
            // 
            this.menuItem5.Checked = false;
            this.menuItem5.DockToRight = false;
            this.menuItem5.Enabled = false;
            this.menuItem5.Image = null;
            this.menuItem5.ImageSelected = null;
            this.menuItem5.Index = 1;
            this.menuItem5.Pressed = false;
            this.menuItem5.Text = "Close View";
            this.menuItem5.Width = 128;
            // 
            // signalBindingSource1
            // 
            this.signalBindingSource1.DataSource = typeof(ABB.Robotics.DataBinding.SignalObject);
            signalObject1.Name = "SignalA";
            signalObject2.Name = "SignalB";
            signalObject3.Name = "SignalC";
            signalObject4.Name = "SignalD";
            signalObject5.Name = "SignalE";
            signalObject6.Name = "SignalF";
            signalObject7.Name = "SignalG";
            signalObject8.Name = "SignalH";
            signalObject9.Name = "SignalI";
            this.signalBindingSource1.Signals.Add(signalObject1);
            this.signalBindingSource1.Signals.Add(signalObject2);
            this.signalBindingSource1.Signals.Add(signalObject3);
            this.signalBindingSource1.Signals.Add(signalObject4);
            this.signalBindingSource1.Signals.Add(signalObject5);
            this.signalBindingSource1.Signals.Add(signalObject6);
            this.signalBindingSource1.Signals.Add(signalObject7);
            this.signalBindingSource1.Signals.Add(signalObject8);
            this.signalBindingSource1.Signals.Add(signalObject9);
            // 
            // groupBox1
            // 
            this.groupBox1.BackColor = System.Drawing.Color.LightGray;
            this.groupBox1.Controls.Add(this.label3);
            this.groupBox1.Controls.Add(this._lblNoSig);
            this.groupBox1.Controls.Add(this.label1);
            this.groupBox1.Controls.Add(this._btnOff);
            this.groupBox1.Controls.Add(this._dataGrid);
            this.groupBox1.Controls.Add(this._btnOn);
            this.groupBox1.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this.groupBox1.HorizontalAlign = ABB.Robotics.Tps.Windows.Forms.HorAlign.Left;
            this.groupBox1.LineStyle = ABB.Robotics.Tps.Windows.Forms.LineStyle.SingleLine;
            this.groupBox1.Location = new System.Drawing.Point(10, 31);
            this.groupBox1.Name = "groupBox1";
            this.groupBox1.Size = new System.Drawing.Size(618, 157);
            this.groupBox1.Title = "Test process signals";
            this.groupBox1.VerticalAlign = ABB.Robotics.Tps.Windows.Forms.VerAlign.Center;
            // 
            // label3
            // 
            this.label3.Font = new System.Drawing.Font("Tahoma", 10F, System.Drawing.FontStyle.Bold);
            this.label3.Location = new System.Drawing.Point(9, 32);
            this.label3.Name = "label3";
            this.label3.Size = new System.Drawing.Size(168, 99);
            this.label3.Text = "Select the signal you want to modify and use the buttons to change the value.";
            // 
            // _lblNoSig
            // 
            this._lblNoSig.Font = new System.Drawing.Font("Tahoma", 10F, System.Drawing.FontStyle.Bold);
            this._lblNoSig.Location = new System.Drawing.Point(458, 12);
            this._lblNoSig.Name = "_lblNoSig";
            this._lblNoSig.Size = new System.Drawing.Size(39, 20);
            this._lblNoSig.Text = "0";
            // 
            // label1
            // 
            this.label1.Font = new System.Drawing.Font("Tahoma", 10F, System.Drawing.FontStyle.Bold);
            this.label1.Location = new System.Drawing.Point(286, 12);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(139, 20);
            this.label1.Text = "Number of signals:";
            // 
            // _btnOff
            // 
            this._btnOff.BackColor = System.Drawing.SystemColors.ActiveBorder;
            this._btnOff.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("_btnOff.BackgroundImage")));
            this._btnOff.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this._btnOff.ForeColor = System.Drawing.Color.DarkRed;
            this._btnOff.Location = new System.Drawing.Point(558, 74);
            this._btnOff.Size = new System.Drawing.Size(48, 40);
            this._btnOff.TabIndex = 11;
            this._btnOff.Text = "";
            this._btnOff.TextAlign = ABB.Robotics.Tps.Windows.Forms.ContentAlignmentABB.MiddleCenter;
            this._btnOff.Click += new System.EventHandler(this._btnOff_Click);
            // 
            // _dataGrid
            // 
            this._dataGrid.BackgroundColor = System.Drawing.Color.FromArgb(((int)(((byte)(128)))), ((int)(((byte)(128)))), ((int)(((byte)(128)))));
            this._dataGrid.DataSource = this.signalBindingSource1;
            this._dataGrid.Location = new System.Drawing.Point(232, 32);
            this._dataGrid.Name = "_dataGrid";
            this._dataGrid.Size = new System.Drawing.Size(292, 118);
            this._dataGrid.TabIndex = 6;
            // 
            // _btnOn
            // 
            this._btnOn.BackColor = System.Drawing.SystemColors.ActiveBorder;
            this._btnOn.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("_btnOn.BackgroundImage")));
            this._btnOn.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this._btnOn.ForeColor = System.Drawing.Color.SeaGreen;
            this._btnOn.Location = new System.Drawing.Point(558, 32);
            this._btnOn.Size = new System.Drawing.Size(48, 40);
            this._btnOn.TabIndex = 9;
            this._btnOn.Text = "";
            this._btnOn.TextAlign = ABB.Robotics.Tps.Windows.Forms.ContentAlignmentABB.MiddleCenter;
            this._btnOn.Click += new System.EventHandler(this._btnOn_Click);
            // 
            // _btnSimOn
            // 
            this._btnSimOn.BackColor = System.Drawing.SystemColors.ActiveBorder;
            this._btnSimOn.BackgroundImage = null;
            this._btnSimOn.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this._btnSimOn.ForeColor = System.Drawing.Color.SeaGreen;
            this._btnSimOn.Location = new System.Drawing.Point(169, 24);
            this._btnSimOn.Size = new System.Drawing.Size(48, 40);
            this._btnSimOn.TabIndex = 10;
            this._btnSimOn.Text = "ON";
            this._btnSimOn.TextAlign = ABB.Robotics.Tps.Windows.Forms.ContentAlignmentABB.MiddleCenter;
            this._btnSimOn.Click += new System.EventHandler(this._btnSimOn_Click);
            // 
            // _btnSimOff
            // 
            this._btnSimOff.BackColor = System.Drawing.SystemColors.ActiveBorder;
            this._btnSimOff.BackgroundImage = null;
            this._btnSimOff.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this._btnSimOff.ForeColor = System.Drawing.Color.DarkRed;
            this._btnSimOff.Location = new System.Drawing.Point(169, 66);
            this._btnSimOff.Size = new System.Drawing.Size(48, 40);
            this._btnSimOff.TabIndex = 12;
            this._btnSimOff.Text = "OFF";
            this._btnSimOff.TextAlign = ABB.Robotics.Tps.Windows.Forms.ContentAlignmentABB.MiddleCenter;
            this._btnSimOff.Click += new System.EventHandler(this._btnSimOff_Click);
            // 
            // label2
            // 
            this.label2.Font = new System.Drawing.Font("Tahoma", 10F, System.Drawing.FontStyle.Bold);
            this.label2.Location = new System.Drawing.Point(9, 127);
            this.label2.Name = "label2";
            this.label2.Size = new System.Drawing.Size(127, 14);
            this.label2.Text = "Simulated mode:";
            // 
            // groupBox2
            // 
            this.groupBox2.BackColor = System.Drawing.Color.LightGray;
            this.groupBox2.Controls.Add(this.label4);
            this.groupBox2.Controls.Add(this._lblSimValue);
            this.groupBox2.Controls.Add(this.label2);
            this.groupBox2.Controls.Add(this._btnSimOff);
            this.groupBox2.Controls.Add(this._btnSimOn);
            this.groupBox2.Controls.Add(this.listView1);
            this.groupBox2.Controls.Add(this._btnStartRout);
            this.groupBox2.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this.groupBox2.HorizontalAlign = ABB.Robotics.Tps.Windows.Forms.HorAlign.Left;
            this.groupBox2.LineStyle = ABB.Robotics.Tps.Windows.Forms.LineStyle.SingleLine;
            this.groupBox2.Location = new System.Drawing.Point(10, 194);
            this.groupBox2.Name = "groupBox2";
            this.groupBox2.Size = new System.Drawing.Size(618, 147);
            this.groupBox2.Title = "Simulate process while executing RAPID";
            this.groupBox2.VerticalAlign = ABB.Robotics.Tps.Windows.Forms.VerAlign.Center;
            // 
            // label4
            // 
            this.label4.Font = new System.Drawing.Font("Tahoma", 10F, System.Drawing.FontStyle.Bold);
            this.label4.Location = new System.Drawing.Point(9, 26);
            this.label4.Name = "label4";
            this.label4.Size = new System.Drawing.Size(154, 93);
            this.label4.Text = "Press ON to switch to simulated mode, then start execution. Press OFF to return t" +
                "o normal mode.";
            // 
            // _lblSimValue
            // 
            this._lblSimValue.Font = new System.Drawing.Font("Tahoma", 12F, System.Drawing.FontStyle.Bold);
            this._lblSimValue.Location = new System.Drawing.Point(142, 119);
            this._lblSimValue.Name = "_lblSimValue";
            this._lblSimValue.Size = new System.Drawing.Size(51, 22);
            // 
            // menuItem4
            // 
            this.menuItem4.Checked = false;
            this.menuItem4.DockToRight = true;
            this.menuItem4.Enabled = true;
            this.menuItem4.Image = null;
            this.menuItem4.ImageSelected = null;
            this.menuItem4.Index = 2;
            this.menuItem4.Pressed = false;
            this.menuItem4.Text = "Close";
            this.menuItem4.Width = 128;
            // 
            // ViewTest
            // 
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Inherit;
            this.BackColor = System.Drawing.Color.LightGray;
            this.Controls.Add(this.groupBox2);
            this.Controls.Add(this.groupBox1);
            // 
            // 
            // 
            this.MainMenu.MenuItems.Add(this.menuItem1);
            this.MainMenu.MenuItems.Add(this.menuItem5);
            this.MainMenu.MenuItems.Add(this.menuItem4);
            this.Size = new System.Drawing.Size(640, 390);
            this.Text = "Test";
            this.Controls.SetChildIndex(this.groupBox1, 0);
            this.Controls.SetChildIndex(this.groupBox2, 0);
            this.groupBox1.ResumeLayout(false);
            this.groupBox2.ResumeLayout(false);
            this.ResumeLayout(false);
        }

        #endregion


        #region Public methods used by the view class to control activation and deactivation of subscriptions to controller events

        /// <summary>
        /// To be executed after constructor & when Test view is active again after Sample Application activation via task bar 
        /// </summary>
        public void Activate()
        {
            // Update label with current value of RAPID data 'simulated'
            UpdateSimulatedStatus();

            // Activate subscription
            if (_isSubscribing == false)
            {
                _rdSimulated.ValueChanged += new DataValueChangedEventHandler(_onSimulated_ValueChanged);
                _isSubscribing = true;
            }
        }

        /// <summary>
        /// To be executed if this view is active at Sample Application deactivation via FP task bar.
        /// Removes any active subscriptions toward robot controller.
        /// </summary>
        public void UnSubscribe()
        {
            if (_isSubscribing == true)
            {
                _rdSimulated.ValueChanged -= new DataValueChangedEventHandler(_onSimulated_ValueChanged);
                _isSubscribing = false;
            }
        }

        #endregion

        #region Event handlers and private methods

        /// <summary>
        /// Executes when value of RAPID data 'simulated' in the controller has changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void _onSimulated_ValueChanged(object sender, DataValueChangedEventArgs e)
        {
            try
            {
                // Enforce thread switch to ensure execution on GUI thread
                this.Invoke(new EventHandler(UpdateSimulatedStatus), sender, e);
            }
            catch (System.Exception ex)
            {
                Debug.Assert(false, "_onSimulated_ValueChanged failed. Message: ", ex.ToString());
            }
        }

        private void UpdateSimulatedStatus(object sender, EventArgs e)
        {
            UpdateSimulatedStatus();
        }

        private void UpdateSimulatedStatus()
        {
            try
            {
                // read RAPID data value
                bool isSimulated = Convert.ToBoolean(_rdSimulated.Value.ToString());

                if (isSimulated == false)
                {
                    _lblSimValue.Text = "OFF";
                    _lblSimValue.ForeColor = Color.DarkRed;
                }
                else
                {
                    _lblSimValue.Text = "ON";
                    _lblSimValue.ForeColor = Color.SeaGreen;
                }
            }
            catch (Exception ex)
            {
                GTPUMessageBox.Show(this.Parent.Parent, null
                    , string.Format("An unexpected error occurred when reading RAPID data. Message {0}", ex.ToString())
                    , "System Error"
                    , System.Windows.Forms.MessageBoxIcon.Hand
                    , System.Windows.Forms.MessageBoxButtons.OK);
            }
        }

        private void _btnSimOn_Click(object sender, EventArgs e)
        {
            try
            {
                IRapidData ird = _rdSimulated.Value;
                ird.FillFromString("true");
                _rdSimulated.Value = ird;
            }
            catch (System.Exception)
            {
                GTPUMessageBox.Show(this, null, "Failed to set RAPID data.", "RAPID data Error");
            }
        }

        private void _btnSimOff_Click(object sender, EventArgs e)
        {
            try
            {
                IRapidData ird = _rdSimulated.Value;
                ird.FillFromString("false");
                _rdSimulated.Value = ird;
            }
            catch (System.Exception)
            {
                GTPUMessageBox.Show(this, null, "Failed to get RAPID data.", "RAPID data Error");
            }
        }

        private void _btnOn_Click(object sender, EventArgs e)
        {
            _tempObj = (ABB.Robotics.DataBinding.SignalObject)signalBindingSource1.Current; //selected signal in the signal binding source
            _tempObj.Value = 1;
        }

        private void _btnOff_Click(object sender, EventArgs e)
        {
            _tempObj = (ABB.Robotics.DataBinding.SignalObject)signalBindingSource1.Current; //selected signal in the signal binding source
            _tempObj.Value = 0;
        }

        /// <summary>
        /// Executes when "Start Selected Routine" is pressed. Sets PP to specified routine (listview-selectedItem).
        /// and starts RAPID execution.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void _btnStartRout_Click(object sender, EventArgs e)
        {
            Task tRob1 = null;

            try
            {
                if (_controller.Rapid.ExecutionStatus != ExecutionStatus.Running)
                {
                    _lvItem = listView1.SelectedItem;
                    if (_lvItem != null)
                    {
                        // Set program pointer to specified routine and start executing
                        string routineName = _lvItem.Text;
                        string moduleName = _lvItem.SubItems[1].Text;
                        tRob1 = _controller.Rapid.GetTask("T_ROB1");
                        tRob1.SetProgramPointer(moduleName, routineName);
                        _controller.Rapid.Start();
                    }
                    else
                    {
                        GTPUMessageBox.Show(this, null
                      , "You must select a routine in the list."
                      , "System Information"
                      , System.Windows.Forms.MessageBoxIcon.Hand
                      , System.Windows.Forms.MessageBoxButtons.OK);
                    }
                }
                else
                {
                    GTPUMessageBox.Show(this, null
                    , "RAPID execution must be stopped before the action can be performed."
                    , "System Information"
                    , System.Windows.Forms.MessageBoxIcon.Hand
                    , System.Windows.Forms.MessageBoxButtons.OK);
                }
            }
            catch (System.Exception)
            {
                GTPUMessageBox.Show(this, null
                , "An error occurred when trying to set program pointer to specified routine and start execution. Check the event log for further information."
                , "System Error"
                , System.Windows.Forms.MessageBoxIcon.Hand
                , System.Windows.Forms.MessageBoxButtons.OK);
            }
            finally
            {
                try
                {
                    if (tRob1 != null)
                    {
                        tRob1.Dispose();
                        tRob1 = null;
                    }
                }
                catch (System.Exception ee)
                {
                    ExceptionHandler.HandleException(this, "_btnStartRout_Click", ee);
                }
            }
        }

        void menuItem4_Click(object sender, EventArgs e)
        {
            this.CloseMe();
        }

        void menuItem6_Click(object sender, EventArgs e)
        {
            // Open Program Data view
            if (_launchService.LaunchView(ABB.Robotics.Tps.Taf.FpStandardView.RapidData, null, false, out this._cookie) != true)
            {
                GTPUMessageBox.Show(this, null, "Could not start Program Data view");
                return;
            }
            else
            {
                // launch succeeded, enable close button
                menuItem5.Enabled = true;
            }
        }

        void menuItem5_Click(object sender, EventArgs e)
        {
            if (_cookie != null)
            {
                _launchService.CloseView(this._cookie);
                _cookie = null;

                // Disable this button
                menuItem5.Enabled = false;
            }
        }

        void menuItem3_Click(object sender, EventArgs e)
        {
            // Open Program Editor
            if (_launchService.LaunchView(ABB.Robotics.Tps.Taf.FpStandardView.RapidEditor, null, false, out this._cookie) != true)
            {
                GTPUMessageBox.Show(this, null, "Could not start Rapid Editor");
                return;
            }
            else
            {
                //launch succeeded, enable close button
                menuItem5.Enabled = true;
            }
        }

        void menuItem2_Click(object sender, EventArgs e)
        {
            // Open Jogging view
            if (_launchService.LaunchView(ABB.Robotics.Tps.Taf.FpStandardView.Jogging, null, false, out this._cookie) != true)
            {
                GTPUMessageBox.Show(this, null, "Could not start Jogging application");
                return;
            }
            else
            {
                //launch succeeded, enable close button
                menuItem5.Enabled = true;
            }
        }

        #endregion


    }
}