using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;

// ABB namespaces
using ABB.Robotics.Tps.Taf;
using ABB.Robotics.Tps.Windows.Forms;
using ABB.Robotics.Tps.Resources;
using ABB.Robotics.Diagnostics;
using ABB.Robotics.Controllers;
using ABB.Robotics.Controllers.RapidDomain;

// Compliance Tool Info 
[assembly: TpsView("TpsViewSampleApp", "tpu-Operator32.gif", "tpu-Operator16.gif", "TpsViewSampleApp.dll", "TpsViewSampleApp.TpsViewSampleApp", StartPanelLocation.Left, TpsViewType.Static, TpsViewStartupTypes.Manual)]
namespace TpsViewSampleApp
{
    /// <summary>
    /// Summary description for TpsViewIRC5App1.
    /// </summary>
    public class TpsViewSampleApp : TpsForm, ITpsViewSetup, ITpsViewActivation
    {
        #region Enums

        private enum ActiveView
        {
            Desktop = 0, // The view represented by this class, (first view with Production and Test buttons)
            Production = 1,
            Test = 2,
        }

        #endregion

        #region Fields

        //layout
        private ABB.Robotics.Tps.Windows.Forms.Button button1; //Production
        private ABB.Robotics.Tps.Windows.Forms.Button button2; //Test

        private ABB.Robotics.Tps.Windows.Forms.TpsLabel tpsLabel1;

        //robot controller object, to be reused throughout the application
        private ABB.Robotics.Controllers.Controller _controller = null;

        //application views 
        private ViewProduction _viewProduction = null;
        private ViewTest _viewTest = null;

        //sets this first view to the currently active view
        private ActiveView _activeView = ActiveView.Desktop;

        //flags
        private bool _isExecuting = false;
        private bool _isInAuto = false;
        private bool _appInFocus = true;

        //needed to launch standard FP apps from this app.
        private ITpsViewLaunchServices _iTpsSite;

        //handles images
        private TpsResourceManager _tpsRm = null;

        #endregion


        #region Constructor
        public TpsViewSampleApp()
        {
            try
            {
                InitializeComponent();
                _tpsRm = new TpsResourceManager();

                // The Install method, which runs after the constructor, creates the robot controller instance and 
                // starts the controller subscriptions needed in this view.
            }
            catch (System.Exception ex)
            {
                // If initialization of application fails a message box is shown
                GTPUMessageBox.Show(this.Parent
                    , null
                    , string.Format("An unexpected error occurred while starting up Sample Application. \n\n{0}", ex.Message)
                    , "Sample Application Start-up Error"
                    , MessageBoxIcon.Hand, MessageBoxButtons.OK);
            }
        }
        #endregion

        #region Dispose

        /// <summary>
        /// Clean up any resources being used.
        /// </summary>
        protected override void Dispose(bool disposing)
        {
            try
            {
                if (!IsDisposed)
                {
                    if (disposing)
                    {
                        // Dispose any open view that may still live (needed in case the user closed the application by tapping
                        // X in the top right corner when either Production or Test view was active)
                        if (_activeView != ActiveView.Desktop)
                        {
                            if (_viewProduction != null)
                            {
                                _viewProduction.Dispose();
                                _viewProduction = null;
                            }

                            if (_viewTest != null)
                            {
                                _viewTest.Dispose();
                                _viewTest = null;
                            }
                        }

                        // Remove any CAPI objects to avoid memory leaks
                        if (_controller != null)
                        {
                            // stop subscriptions to controller events
                            _controller.OperatingModeChanged -= new OperatingModeChangedEventHandler(_onOperatingModeChanged);
                            _controller.Rapid.ExecutionStatusChanged -= new ExecutionStatusChangedEventHandler(_onExecutionStatusChanged);

                            _controller.Dispose();
                            _controller = null;
                        }

                        if (_tpsRm != null)
                        {
                            _tpsRm = null;
                        }

                    }
                    // base class should dispose of any controls added to the control collection of this class
                    base.Dispose(disposing);
                }
            }
            catch (System.Exception ex)
            {
                GTPUMessageBox.Show(this.Parent.Parent, null
                    , "An unexpected error occurred while closing down the application.\n\nError message: " + ex.Message
                    , "Application Error"
                    , MessageBoxIcon.Hand, MessageBoxButtons.OK);
            }
        }

        #endregion

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(TpsViewSampleApp));
            this.tpsLabel1 = new ABB.Robotics.Tps.Windows.Forms.TpsLabel();
            this.button1 = new ABB.Robotics.Tps.Windows.Forms.Button();
            this.button2 = new ABB.Robotics.Tps.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // tpsLabel1
            // 
            this.tpsLabel1.BackColor = System.Drawing.Color.LightGray;
            this.tpsLabel1.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this.tpsLabel1.ForeColor = System.Drawing.SystemColors.ActiveCaption;
            this.tpsLabel1.Location = new System.Drawing.Point(112, 63);
            this.tpsLabel1.Size = new System.Drawing.Size(414, 61);
            this.tpsLabel1.TabIndex = 0;
            this.tpsLabel1.TextAlignment = System.Drawing.ContentAlignment.TopCenter;
            this.tpsLabel1.Title = "Sample Application";
            // 
            // button1
            // 
            this.button1.BackColor = System.Drawing.Color.White;
            this.button1.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("button1.BackgroundImage")));
            this.button1.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this.button1.Location = new System.Drawing.Point(153, 160);
            this.button1.Size = new System.Drawing.Size(120, 120);
            this.button1.TabIndex = 1;
            this.button1.Text = "";
            this.button1.TextAlign = ABB.Robotics.Tps.Windows.Forms.ContentAlignmentABB.MiddleCenter;
            this.button1.Click += new System.EventHandler(this.button1_Click);
            // 
            // button2
            // 
            this.button2.BackColor = System.Drawing.Color.White;
            this.button2.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("button2.BackgroundImage")));
            this.button2.Font = ABB.Robotics.Tps.Windows.Forms.TpsFont.Font10b;
            this.button2.Location = new System.Drawing.Point(363, 160);
            this.button2.Size = new System.Drawing.Size(120, 120);
            this.button2.TabIndex = 2;
            this.button2.Text = "";
            this.button2.TextAlign = ABB.Robotics.Tps.Windows.Forms.ContentAlignmentABB.MiddleCenter;
            this.button2.Click += new System.EventHandler(this.button2_Click);
            // 
            // TpsViewSampleApp
            // 
            this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Inherit;
            this.BackColor = System.Drawing.Color.LightGray;
            this.Controls.Add(this.button2);
            this.Controls.Add(this.button1);
            this.Controls.Add(this.tpsLabel1);
            this.Size = new System.Drawing.Size(640, 390);
            this.ResumeLayout(false);
        }
        #endregion

        #region Event handlers

        /// <summary>
        /// Executes when Production button is pressed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                // Wait cursor if it is performance demanding to open the view...
                Cursor.Current = Cursors.WaitCursor;

                // Set active view 
                _activeView = ActiveView.Production;

                // Create view
                _viewProduction = new ViewProduction(_controller, _tpsRm);

                // Set up subscription to Closing event of Production view
                _viewProduction.Closing += new System.ComponentModel.CancelEventHandler(_onViewClosing);
                _viewProduction.Closed += new EventHandler(_viewClosed);
                _viewProduction.ShowMe(this);

                // Ask Production view to set up its subscriptions to controller events
                _viewProduction.Activate();
            }
            catch (System.Exception ex)
            {
                DisplayErrorMessage(ex.Message);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        /// <summary>
        /// Executes when Test button is pressed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                Cursor.Current = Cursors.WaitCursor;
                _activeView = ActiveView.Test;
                _viewTest = new ViewTest(_controller, _iTpsSite);
                _viewTest.Closing += new System.ComponentModel.CancelEventHandler(_onViewClosing);
                _viewTest.Closed += new EventHandler(_viewClosed);
                _viewTest.ShowMe(this);
                _viewTest.Activate();
            }
            catch (System.Exception ex)
            {
                DisplayErrorMessage(ex.Message);
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }

        /// <summary>
        /// Executes when Close button of Production/Test view is pressed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="args"></param>
        private void _onViewClosing(object sender, System.ComponentModel.CancelEventArgs args)
        {
            try
            {
                // Set active view
                _activeView = ActiveView.Desktop;

                // Enable/disable Test button
                if (_isInAuto || _isExecuting)
                {
                    button2.Enabled = false;
                    button2.BackgroundImage = _tpsRm.GetImage("DisabledTestBtn.gif");
                }
                else
                {
                    button2.Enabled = true;
                    button2.BackgroundImage = _tpsRm.GetImage("TestBtn_Up.gif");
                }
            }
            catch (System.Exception ex)
            {
                Debug.Assert(false, "_onViewClosing failed with message: ", ex.ToString());
            }
        }

        void _viewClosed(object sender, EventArgs e)
        {
            if (sender is Control)
            {
                ((Control)sender).Dispose();
            }
        }

        /// <summary>
        /// Executes when robot controller operating mode has changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void _onOperatingModeChanged(object sender, OperatingModeChangedEventArgs e)
        {
            try
            {
                if (e.NewOperatingMode == ControllerOperatingMode.Auto)
                {
                    _isInAuto = true;
                }
                else
                {
                    _isInAuto = false;
                }
                // If this view is active, enforce switch to GUI thread by the use of Invoke 
                // before updating the user interface
                if (_appInFocus && _activeView == ActiveView.Desktop)
                {
                    this.Invoke(new EventHandler(UpdateGUI), sender, e);
                }
            }
            catch (System.Exception ex)
            {
                // Debug.Assert(false, "_onOperatingModeChanged failed. Message: ", ex.ToString());
                ExceptionHandler.HandleException("TpsViewSampleApp", "_onOperatingModeChanged", ex);
            }
        }

        /// <summary>
        /// Executes when robot controller execution status has changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void _onExecutionStatusChanged(object sender, ExecutionStatusChangedEventArgs e)
        {
            try
            {
                if (e.NewStatus == ExecutionStatus.Running)
                {
                    _isExecuting = true;
                }
                else
                {
                    _isExecuting = false;
                }
                // If this view is active, enforce switch to GUI thread by the use of Invoke 
                // before updating the user interface
                if (_appInFocus && _activeView == ActiveView.Desktop)
                {
                    this.Invoke(new EventHandler(UpdateGUI), sender, e);
                }
            }
            catch (System.Exception ex)
            {
                // Debug.Assert(false, "_onExecutionStatusChanged failed. Message: ", ex.ToString());
                ExceptionHandler.HandleException("TpsViewSampleApp", "_onExecutionStatusChanged", ex);
            }
        }

        /// <summary>
        /// This delegate method updates the GUI. It is guaranteed to execute on the GUI thread.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void UpdateGUI(object sender, EventArgs e)
        {
            try
            {
                if (_isExecuting || _isInAuto) // have been set by calling method
                {
                    button2.Enabled = false;
                    button2.BackgroundImage = _tpsRm.GetImage("DisabledTestBtn.gif");
                }
                else
                {
                    button2.Enabled = true;
                    button2.BackgroundImage = _tpsRm.GetImage("TestBtn_Up.gif");
                }
            }
            catch (System.Exception ex)
            {
                // ExceptionHandler.ReThrowException("TpsViewSampleApp", "UpdateGUI", ex);
                ExceptionHandler.HandleException("TpsViewSampleApp", "UpdateGUI", ex);
            }
        }

        #endregion

        #region Private methods

        private void DisplayErrorMessage(string message)
        {
            string triedView = _activeView.ToString();

            // Reset Active view
            _activeView = ActiveView.Desktop;

            // Show GTPUMessageBox on exception
            GTPUMessageBox.Show(this.Parent.Parent, null
                , string.Format("Unable to open {0} view. \n\nDo the preparations described in 'Prepare signals and RAPID data.doc' and try open the view again.\n\nError message: {1}", triedView, message)
                , string.Format("{0}Start-up Error", triedView)
                , MessageBoxIcon.Hand, MessageBoxButtons.OK);
        }

        #endregion

        #region ITpsViewSetup Members

        /// <summary>
        /// This method is called by TAF when the control is closed down (before Dispose is called).
        /// </summary>	
        void ITpsViewSetup.Uninstall()
        {
            // TODO:  Add TpsViewSampleApp.Uninstall implementation
        }

        /// <summary>
        /// This method is called by TAF when the control is installed in the framework.
        /// </summary>				
        bool ITpsViewSetup.Install(object sender, object data)
        {
            try
            {
                // Create controller instance and check operating and execution mode
                _controller = new Controller();

                if (_controller.OperatingMode == ControllerOperatingMode.Auto)
                {
                    _isInAuto = true;
                }

                if (_controller.Rapid.ExecutionStatus == ExecutionStatus.Running)
                {
                    _isExecuting = true;
                }

                // Start subscriptions
                _controller.OperatingModeChanged += new OperatingModeChangedEventHandler(_onOperatingModeChanged);
                _controller.Rapid.ExecutionStatusChanged += new ExecutionStatusChangedEventHandler(_onExecutionStatusChanged);

                // Save the sender object to be able to later use the ITpsViewLaunchServices interface
                if (sender is ITpsViewLaunchServices)
                {
                    this._iTpsSite = sender as ITpsViewLaunchServices;
                    return true;
                }
            }
            catch (System.Exception ex)
            {
                ExceptionHandler.ReThrowException("TpsViewSampleApp", "ITpsViewActivation.Activate", ex);
                //ExceptionHandler.HandleException("TpsViewSampleApp", "ITpsViewSetup.Install", ex);
            }
            return false;
        }

        #endregion

        #region ITpsViewActivation Members

        /// <summary>
        /// Executes when application gets out of focus
        /// </summary>
        void ITpsViewActivation.Deactivate()
        {
            try
            {
                // Stop any subscriptions to controller events that the active view may have activated
                if (_activeView == ActiveView.Production)
                {
                    _viewProduction.UnSubscribe();
                }
                else if (_activeView == ActiveView.Test)
                {
                    _viewTest.UnSubscribe();
                }

                _appInFocus = false;
            }
            catch (System.Exception ex)
            {
                Debug.Assert(false, "ITpsViewActivation.Deactivate failed. Message: ", ex.ToString());
            }
        }

        /// <summary>
        /// Executes when application gets focus
        /// </summary>
        void ITpsViewActivation.Activate()
        {
            try
            {
                if (_activeView == ActiveView.Desktop) // If first view is active
                {
                    // Enable/disable Test button
                    if (_isInAuto || _isExecuting)
                    {
                        button2.Enabled = false;
                        button2.BackgroundImage = _tpsRm.GetImage("DisabledTestBtn.gif");
                    }
                    else
                    {
                        button2.Enabled = true;
                        button2.BackgroundImage = _tpsRm.GetImage("TestBtn_Up.gif");
                    }
                }
                else if (_activeView == ActiveView.Production)
                {
                    _viewProduction.Activate();
                }
                else if (_activeView == ActiveView.Test)
                {
                    _viewTest.Activate();
                }

                _appInFocus = true;

            }
            catch (System.Exception ex)
            {
                ExceptionHandler.ReThrowException("TpsViewSampleApp", "ITpsViewActivation.Activate", ex);
            }
        }

        #endregion

    }
}
